# Generated by Django 2.0.7 on 2018-07-26 16:49

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='GlobeeIPN',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('payment_status', models.CharField(choices=[('unpaid', 'unpaid - ready to receive payment'), ('paid', 'waiting for confirmations'), ('underpaid', 'user has paid less'), ('overpaid', 'user has mistakenly paid more'), ('paid_late', 'the payment was made outside of the quotation window'), ('confirmed', 'payment has been confirmed'), ('completed', 'payment-request is now completed')], help_text='Globee payment status', max_length=12)),
                ('payment_id', models.CharField(help_text='Globee payment ID', max_length=255)),
                ('total', models.FloatField(help_text='The amount in fiat or crypto-currency')),
                ('customer_email', models.EmailField(help_text='The email address of your customer', max_length=254)),
                ('customer_name', models.CharField(blank=True, help_text='The name of your customer', max_length=50, null=True)),
                ('currency', models.CharField(default='USD', help_text='ISO 4217 currency codes', max_length=3)),
                ('custom_payment_id', models.CharField(blank=True, help_text='A reference or custom identifier that you can use to link the payment back to your system', max_length=255, null=True)),
                ('callback_data', models.CharField(blank=True, help_text='Passthrough data that will be returned in the IPN callback', max_length=255, null=True)),
                ('notification_email', models.CharField(blank=True, help_text='An email address that the system will send a notification email to once the payment has been confirmed', max_length=255, null=True)),
                ('confirmation_speed', models.CharField(choices=[('low', 'low speed / low risk'), ('medium', 'medium speed / medium risk'), ('high', 'high speed with / risk')], default='medium', max_length=5)),
                ('created_at', models.DateTimeField()),
                ('expires_at', models.DateTimeField()),
            ],
        ),
    ]



from django.conf import settings
from django.core.exceptions import ValidationError
import requests
from django.core.validators import validate_email


class GlobeePayment:

    data = dict()
    redirect_url = None
    test_mode = True
    auth_key = None
    api_url = 'https://globee.com/payment-api/v1'
    headers = None

    def __init__(self, data={}):
        self.data = data
        self.test_mode = getattr(settings, 'GLOBEE_TEST_MODE', True)
        self.auth_key = getattr(settings, 'GLOBEE_AUTH_KEY', None)
        if self.auth_key is None:
            raise ValidationError('GLOBEE_AUTH_KEY not found!')
        if not isinstance(self.auth_key, str):
            raise ValidationError('GLOBEE_AUTH_KEY is not a string!')
        if self.test_mode:
            self.api_url = 'https://test.globee.com/payment-api/v1'
        self.headers = {
            'Accept': 'application/json',
            'X-AUTH-KEY': self.auth_key
        }

    def ping(self):
        r = requests.get('%s/ping' % self.api_url, headers=self.headers, json=self.data)
        response = r.json()
        if r.status_code == 200:
            if response['success']:
                return True
            raise ValidationError('result: %s' % response['result'])
        raise ValidationError("status code %s: %s" % (r.status_code, response['message']))

    def check_required_fields(self):
        try:
            total = self.data['total']
            email = self.data['customer']['email']
            if not isinstance(total, (int, float)):
                raise ValidationError('total is not a int or float!')
        except KeyError as e:
            raise KeyError(e)
        try:
            validate_email(self.data['customer']['email'])
        except ValidationError as e:
            raise ValidationError(e)
        return True

    def create_request(self):
        r = requests.post('%s/payment-request' % self.api_url, headers=self.headers, json=self.data)
        response = r.json()
        if r.status_code == 200:
            self.redirect_url = response['data']['redirect_url']
            return True
        raise ValidationError('status code: %s - %s' % (r.status_code, response))

    def get_payment_url(self):
        return self.redirect_url

    def get_payment_by_id(self, payment_id):
        r = requests.get('%s/payment-request/%s' % (self.api_url, payment_id), headers=self.headers, json=self.data)
        response = r.json()
        if r.status_code == 200:
            return response['data']
        raise ValidationError('status code: %s - %s' % (r.status_code, response))



from django.contrib import admin

from globee.models import GlobeeIPN


class GlobeeIPNAdmin(admin.ModelAdmin):
    list_display = ('payment_id', 'payment_status', 'total', 'currency', 'custom_payment_id', 'created_at')
    date_hierarchy = 'created_at'
    search_fields = ('payment_id', 'custom_payment_id')


admin.site.register(GlobeeIPN, GlobeeIPNAdmin)



from django.db import models

from globee.signals import globee_valid_ipn

PAYMENT_STATUS_GLOBEE_UNPAID = 'unpaid'
PAYMENT_STATUS_GLOBEE_PAID = 'paid'
PAYMENT_STATUS_GLOBEE_UNDERPAID = 'underpaid'
PAYMENT_STATUS_GLOBEE_OVERPAID = 'overpaid'
PAYMENT_STATUS_GLOBEE_PAID_LATE = 'paid_late'
PAYMENT_STATUS_GLOBEE_CONFIRMED = 'confirmed'
PAYMENT_STATUS_GLOBEE_COMPLETED = 'completed'

SPEED_STATUS_GLOBEE_LOW = 'low'
SPEED_STATUS_GLOBEE_MEDIUM = 'medium'
SPEED_STATUS_GLOBEE_HIGH = 'high'


class GlobeeIPN(models.Model):

    PAYMENT_STATUS_CHOICES = (
        (PAYMENT_STATUS_GLOBEE_UNPAID, 'unpaid - ready to receive payment'),
        (PAYMENT_STATUS_GLOBEE_PAID, 'waiting for confirmations'),
        (PAYMENT_STATUS_GLOBEE_UNDERPAID, 'user has paid less'),
        (PAYMENT_STATUS_GLOBEE_OVERPAID, 'user has mistakenly paid more'),
        (PAYMENT_STATUS_GLOBEE_PAID_LATE, 'the payment was made outside of the quotation window'),
        (PAYMENT_STATUS_GLOBEE_CONFIRMED, 'payment has been confirmed'),
        (PAYMENT_STATUS_GLOBEE_COMPLETED, 'payment-request is now completed'),
    )

    SPEED_STATUS_CHOICES = (
        (SPEED_STATUS_GLOBEE_LOW, 'low speed / low risk'),
        (SPEED_STATUS_GLOBEE_MEDIUM, 'medium speed / medium risk'),
        (SPEED_STATUS_GLOBEE_HIGH, 'high speed with / risk'),
    )

    payment_status = models.CharField(max_length=12, choices=PAYMENT_STATUS_CHOICES, help_text='Globee payment status')
    payment_id = models.CharField(max_length=255, help_text='Globee payment ID')
    total = models.FloatField(help_text='The amount in fiat or crypto-currency')
    customer_email = models.EmailField(help_text='The email address of your customer')
    customer_name = models.CharField(max_length=50, null=True, blank=True, help_text='The name of your customer')
    currency = models.CharField(max_length=3, default='USD', help_text='ISO 4217 currency codes')
    custom_payment_id = models.CharField(max_length=255, blank=True, null=True, help_text='A reference or custom identifier that you can use to link the payment back to your system')
    callback_data = models.CharField(max_length=255, blank=True, null=True, help_text='Passthrough data that will be returned in the IPN callback')
    notification_email = models.CharField(max_length=255, blank=True, null=True, help_text='An email address that the system will send a notification email to once the payment has been confirmed')
    confirmation_speed = models.CharField(max_length=50, choices=SPEED_STATUS_CHOICES, default=SPEED_STATUS_GLOBEE_MEDIUM)
    created_at = models.DateTimeField()
    expires_at = models.DateTimeField()

    def send_valid_signal(self):
        globee_valid_ipn.send(sender=self)



import json

from django.core.exceptions import ValidationError
from django.test import TestCase, override_settings, Client
from django.urls import reverse

from globee.core import GlobeePayment
from globee.models import GlobeeIPN


class GlobeePingTestCase(TestCase):

    def test_ping_valid(self):
        globee_payment = GlobeePayment()
        self.assertTrue(globee_payment.ping())

    @override_settings(GLOBEE_AUTH_KEY="INVALID_KEY")
    def test_ping_invalid_key(self):
        globee_payment = GlobeePayment()
        with self.assertRaises(ValidationError):
            globee_payment.ping()


class GlobeeRequiredFieldsTestCase(TestCase):

    def test_required_fields_valid(self):
        data = {
            'total': 13.37,
            'customer': {
                'name': 'foobar',
                'email': 'foobar@example.com'
            },
        }
        globee_payment = GlobeePayment(data=data)
        self.assertTrue(globee_payment.check_required_fields())

    def test_required_fields_invalid_total(self):
        data = {
            'total': 'foobar',
            'customer': {
                'email': 'foobar@example.com'
            },
        }
        globee_payment = GlobeePayment(data=data)
        with self.assertRaises(ValidationError):
            globee_payment.check_required_fields()

    def test_required_fields_invalid_email(self):
        data = {
            'total': 13.37,
            'customer': {
                'email': 'invalid_mail.com'
            },
        }
        globee_payment = GlobeePayment(data=data)
        with self.assertRaises(ValidationError):
            globee_payment.check_required_fields()

    def test_required_fields_invalid_total_not_set(self):
        data = {
            'customer': {
                'email': 'foobar@example.com'
            },
        }
        globee_payment = GlobeePayment(data=data)
        with self.assertRaises(KeyError):
            globee_payment.check_required_fields()

    def test_required_fields_invalid_email_not_set(self):
        data = {
            'total': 13.37,
        }
        globee_payment = GlobeePayment(data=data)
        with self.assertRaises(KeyError):
            globee_payment.check_required_fields()


@override_settings(GLOBEE_TEST_MODE=True)
class GlobeeCreatePaymentTestCase(TestCase):

    def test_create_payment_valid(self):
        data = {
            'total': 13.37,
            'currency': 'EUR',
            'customer': {
                'name': 'foobar',
                'email': 'foobar@example.com'
            },
        }
        globee_payment = GlobeePayment(data=data)
        self.assertTrue(globee_payment.check_required_fields())
        self.assertTrue(globee_payment.create_request())

    def test_create_payment_invalid_empty_data(self):
        globee_payment = GlobeePayment()
        with self.assertRaises(KeyError):
            globee_payment.check_required_fields()
        with self.assertRaises(ValidationError):
            globee_payment.create_request()

    @override_settings(GLOBEE_AUTH_KEY="INVALID_KEY")
    def test_create_payment_invalid_key(self):
        data = {
            'total': 13.37,
            'customer': {
                'name': 'foobar',
                'email': 'foobar@example.com'
            },
        }
        globee_payment = GlobeePayment(data=data)
        self.assertTrue(globee_payment.check_required_fields())
        with self.assertRaises(ValidationError):
            globee_payment.create_request()

    def test_create_payment_invalid_email_not_set(self):
        data = {
            'total': 13.37,
            'customer': {
                'name': 'foobar',
            },
        }
        globee_payment = GlobeePayment(data=data)
        with self.assertRaises(ValidationError):
            globee_payment.create_request()

    def test_create_payment_invalid_urls(self):
        data = {
            'total': 13.37,
            'customer': {
                'email': 'foobar@example.com'
            },
            'success_url': 'invalid-url',
            'cancel_url': 'invalid-url',
            'ipn_url': 'invalid-url',
        }
        globee_payment = GlobeePayment(data=data)
        self.assertTrue(globee_payment.check_required_fields())
        with self.assertRaises(ValidationError):
            globee_payment.create_request()


@override_settings(GLOBEE_TEST_MODE=True)
class GlobeePaymentIPNTestCase(TestCase):

    def setUp(self):
        GlobeeIPN.objects.all().delete()
        self.assertEqual(0, GlobeeIPN.objects.count())

    @override_settings(ROOT_URLCONF='globee.urls')
    def test_ipn_view_valid(self):
        count_before = 0
        self.assertEqual(count_before, GlobeeIPN.objects.all().count())
        payment_data = {
            "id": "a1B2c3D4e5F6g7H8i9J0kL",
            "status": "paid",
            "total": "123.45",
            "currency": "USD",
            "custom_payment_id": "742",
            "callback_data": "example data",
            "customer": {
                "name": "John Smit",
                "email": "john.smit@example.com"
            },
            "redirect_url": "http:\/\/globee.com\/invoice\/a1B2c3D4e5F6g7H8i9J0kL",
            "success_url": "https:\/\/www.example.com/success",
            "cancel_url": "https:\/\/www.example.com/cancel",
            "ipn_url": "https:\/\/www.example.com/globee/ipn-callback",
            "notification_email": None,
            "confirmation_speed": "medium",
            "expires_at": "2018-01-25 12:31:04",
            "created_at": "2018-01-25 12:16:04"
        }
        client = Client()
        client.generic('POST', reverse('globee-ipn'), bytes(json.dumps(payment_data), 'utf-8'))
        self.assertEqual(count_before + 1, GlobeeIPN.objects.all().count())



import json
import logging
import datetime

import pytz
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST

from globee.models import GlobeeIPN


logger = logging.getLogger(__name__)


@require_POST
@csrf_exempt
def globee_ipn_view(request):
    payment_data = json.loads(request.body.decode("utf-8"))
    logger.debug("Globee POST data: %s", payment_data)
    if 'id' in payment_data:
        created_at = datetime.datetime.strptime(payment_data['created_at'], '%Y-%m-%d %H:%M:%S')
        expires_at = datetime.datetime.strptime(payment_data['expires_at'], '%Y-%m-%d %H:%M:%S')
        defaults = {
            'payment_status': payment_data['status'],
            'total': float(payment_data['total']),
            'currency': payment_data['currency'],
            'custom_payment_id': payment_data['custom_payment_id'],
            'callback_data': payment_data['callback_data'],
            'customer_email': payment_data['customer']['email'],
            'customer_name': payment_data['customer']['name'],
            'created_at': pytz.utc.localize(created_at),
            'expires_at': pytz.utc.localize(expires_at),
        }
        payment, created = GlobeeIPN.objects.update_or_create(payment_id=payment_data['id'], defaults=defaults)
        payment.send_valid_signal()
    return HttpResponse("Ok")



# Generated by Django 2.0.7 on 2018-07-27 00:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('globee', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='globeeipn',
            name='confirmation_speed',
            field=models.CharField(choices=[('low', 'low speed / low risk'), ('medium', 'medium speed / medium risk'), ('high', 'high speed with / risk')], default='medium', max_length=50),
        ),
    ]
